import { Transactionable } from '@nocobase/database';
import { Plugin } from '@nocobase/server';
import { Registry } from '@nocobase/utils';
import { Logger } from '@nocobase/logger';
import Processor from './Processor';
import { CustomFunction } from './functions';
import Trigger from './triggers';
import { InstructionInterface } from './instructions';
import type { ExecutionModel, WorkflowModel } from './types';
type ID = number | string;
export default class PluginWorkflowServer extends Plugin {
    instructions: Registry<InstructionInterface>;
    triggers: Registry<Trigger>;
    functions: Registry<CustomFunction>;
    private ready;
    private executing;
    private pending;
    private events;
    private eventsCount;
    private loggerCache;
    private meter;
    getLogger(workflowId: ID): Logger;
    isWorkflowSync(workflow: WorkflowModel): boolean;
    onBeforeSave: (instance: WorkflowModel, options: any) => Promise<void>;
    registerTrigger<T extends Trigger>(type: string, trigger: T | {
        new (p: Plugin): T;
    }): void;
    registerInstruction(type: string, instruction: InstructionInterface | {
        new (p: Plugin): InstructionInterface;
    }): void;
    private initTriggers;
    private initInstructions;
    load(): Promise<void>;
    toggle(workflow: WorkflowModel, enable?: boolean): void;
    trigger(workflow: WorkflowModel, context: object, options?: {
        [key: string]: any;
    } & Transactionable): void | Promise<Processor | null>;
    private triggerSync;
    resume(job: any): Promise<void>;
    createProcessor(execution: ExecutionModel, options?: {}): Processor;
    private createExecution;
    private prepare;
    private dispatch;
    private process;
}
export {};
