var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var CollectionTrigger_exports = {};
__export(CollectionTrigger_exports, {
  default: () => CollectionTrigger
});
module.exports = __toCommonJS(CollectionTrigger_exports);
var import__ = __toESM(require("."));
var import_utils = require("../utils");
const MODE_BITMAP = {
  CREATE: 1,
  UPDATE: 2,
  DESTROY: 4
};
const MODE_BITMAP_EVENTS = /* @__PURE__ */ new Map();
MODE_BITMAP_EVENTS.set(MODE_BITMAP.CREATE, "afterCreateWithAssociations");
MODE_BITMAP_EVENTS.set(MODE_BITMAP.UPDATE, "afterUpdateWithAssociations");
MODE_BITMAP_EVENTS.set(MODE_BITMAP.DESTROY, "afterDestroy");
function getHookId(workflow, type) {
  return `${type}#${workflow.id}`;
}
function getFieldRawName(collection, name) {
  const field = collection.getField(name);
  if (field && field.type === "belongsTo") {
    return field.foreignKey;
  }
  return name;
}
async function handler(workflow, data, options) {
  var _a;
  const { collection: collectionName, condition, changed, mode, appends } = workflow.config;
  const collection = data.constructor.database.getCollection(collectionName);
  const { transaction, context } = options;
  const { repository, model } = collection;
  if (changed && changed.length && changed.filter((name) => !["linkTo", "hasOne", "hasMany", "belongsToMany"].includes(collection.getField(name).type)).every((name) => !data.changedWithAssociations(getFieldRawName(collection, name)))) {
    return;
  }
  if (condition && ((_a = condition.$and) == null ? void 0 : _a.length)) {
    const count = await repository.count({
      filter: {
        $and: [condition, { [model.primaryKeyAttribute]: data[model.primaryKeyAttribute] }]
      },
      context,
      transaction
    });
    if (!count) {
      return;
    }
  }
  let result = data;
  if ((appends == null ? void 0 : appends.length) && !(mode & MODE_BITMAP.DESTROY)) {
    const includeFields = appends.reduce((set, field) => {
      set.add(field.split(".")[0]);
      set.add(field);
      return set;
    }, /* @__PURE__ */ new Set());
    result = await repository.findOne({
      filterByTk: data[model.primaryKeyAttribute],
      appends: Array.from(includeFields),
      transaction
    });
  }
  const json = (0, import_utils.toJSON)(result);
  if (workflow.sync) {
    await this.workflow.trigger(
      workflow,
      { data: json, stack: context == null ? void 0 : context.stack },
      {
        transaction
      }
    );
  } else {
    this.workflow.trigger(workflow, { data: json, stack: context == null ? void 0 : context.stack });
  }
}
class CollectionTrigger extends import__.default {
  events = /* @__PURE__ */ new Map();
  on(workflow) {
    const { db } = this.workflow.app;
    const { collection, mode } = workflow.config;
    const Collection2 = db.getCollection(collection);
    if (!Collection2) {
      return;
    }
    for (const [key, type] of MODE_BITMAP_EVENTS.entries()) {
      const event = `${collection}.${type}`;
      const name = getHookId(workflow, event);
      if (mode & key) {
        if (!this.events.has(name)) {
          const listener = handler.bind(this, workflow);
          this.events.set(name, listener);
          db.on(event, listener);
        }
      } else {
        const listener = this.events.get(name);
        if (listener) {
          db.off(event, listener);
          this.events.delete(name);
        }
      }
    }
  }
  off(workflow) {
    const { db } = this.workflow.app;
    const { collection, mode } = workflow.config;
    const Collection2 = db.getCollection(collection);
    if (!Collection2) {
      return;
    }
    for (const [key, type] of MODE_BITMAP_EVENTS.entries()) {
      const event = `${collection}.${type}`;
      const name = getHookId(workflow, event);
      if (mode & key) {
        const listener = this.events.get(name);
        if (listener) {
          db.off(event, listener);
          this.events.delete(name);
        }
      }
    }
  }
  async validateEvent(workflow, context, options) {
    if (context.stack) {
      const existed = await workflow.countExecutions({
        where: {
          id: context.stack
        },
        transaction: options.transaction
      });
      if (existed) {
        this.workflow.getLogger(workflow.id).warn(
          `workflow ${workflow.id} has already been triggered in stack executions (${context.stack}), and newly triggering will be skipped.`
        );
        return false;
      }
    }
    return true;
  }
}
